##############################################################################
#
# Copyright (c) 2006-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from zope.component.hooks import getSite  # noqa: F401
from grokcore.component import *  # noqa: F401, F403
from grokcore.site.directive import site  # noqa: F401
from grokcore.site.directive import local_utility  # noqa: F401
from grokcore.site.directive import install_on  # noqa: F401
from grokcore.site.components import Site  # noqa: F401
from grokcore.site.components import LocalUtility  # noqa: F401
from grokcore.site.components import Application  # noqa: F401
from grokcore.site.util import getApplication  # noqa: F401

import grokcore.site.testing  # noqa: F401

from grokcore.site.interfaces import IApplication  # noqa: F401
from grokcore.site.interfaces import IApplicationAddedEvent  # noqa: F401
from grokcore.site.interfaces import ApplicationAddedEvent  # noqa: F401

from grokcore.site.interfaces import IGrokcoreSiteAPI  # noqa: F401
__all__ = list(IGrokcoreSiteAPI)



"""
Local Utilities can be registered on subclasses of grok.Site using
grok.local_utility:

  >>> cave = Cave()
  >>> getRootFolder()["cave"] = cave

  >>> from zope import component
  >>> from zope.component.hooks import getSite, setSite
  >>> setSite(cave)

  >>> fireplace = component.getUtility(IFireplace)
  >>> IFireplace.providedBy(fireplace)
  True
  >>> isinstance(fireplace, Fireplace)
  True

  >>> club = component.getUtility(IClub)
  >>> IClub.providedBy(club)
  True
  >>> isinstance(club, Club)
  True

  >>> spiky = component.getUtility(IClub, name='spiky')
  >>> IClub.providedBy(spiky)
  True
  >>> isinstance(spiky, SpikyClub)
  True

  >>> mammoth = component.getUtility(IMammoth)
  >>> IMammoth.providedBy(mammoth)
  True
  >>> isinstance(mammoth, Mammoth)
  True

  >>> tiger = component.getUtility(IMammoth, name='tiger')
  >>> IMammoth.providedBy(tiger)
  True
  >>> isinstance(tiger, SabretoothTiger)
  True

  >>> painting = component.getUtility(IPainting, name='blackandwhite')
  >>> IPainting.providedBy(painting)
  True
  >>> isinstance(painting, CavePainting)
  True

  >>> colored = component.getUtility(IPainting, name='color')
  >>> IPainting.providedBy(colored)
  True
  >>> isinstance(colored, ColoredCavePainting)
  True

Since it is a local utility, it is not available outside its site:

  >>> setSite(None)
  >>> component.getUtility(IFireplace)
  Traceback (most recent call last):
    ...
  zope.interface.interfaces.ComponentLookupError: (<InterfaceClass grokcore.site.tests.functional.utility.local.IFireplace>, '')

  >>> component.getUtility(IClub)
  Traceback (most recent call last):
    ...
  zope.interface.interfaces.ComponentLookupError: (<InterfaceClass grokcore.site.tests.functional.utility.local.IClub>, '')

  >>> component.getUtility(IClub, name='spiky')
  Traceback (most recent call last):
    ...
  zope.interface.interfaces.ComponentLookupError: (<InterfaceClass grokcore.site.tests.functional.utility.local.IClub>, 'spiky')

  >>> component.getUtility(IMammoth)
  Traceback (most recent call last):
    ...
  zope.interface.interfaces.ComponentLookupError: (<InterfaceClass grokcore.site.tests.functional.utility.local.IMammoth>, '')

  >>> component.getUtility(IMammoth, name='tiger')
  Traceback (most recent call last):
    ...
  zope.interface.interfaces.ComponentLookupError: (<InterfaceClass grokcore.site.tests.functional.utility.local.IMammoth>, 'tiger')

  >>> component.getUtility(IPainting, name='blackandwhite')
  Traceback (most recent call last):
    ...
  zope.interface.interfaces.ComponentLookupError: (<InterfaceClass grokcore.site.tests.functional.utility.local.IPainting>, 'blackandwhite')

  >>> component.getUtility(IPainting, name='color')
  Traceback (most recent call last):
    ...
  zope.interface.interfaces.ComponentLookupError: (<InterfaceClass grokcore.site.tests.functional.utility.local.IPainting>, 'color')
"""  # noqa: E501
import grokcore.site
from zope import interface
import persistent


class IFireplace(interface.Interface):
    pass


class IClub(interface.Interface):
    pass


class ISpiky(interface.Interface):
    pass


class IMammoth(interface.Interface):
    pass


@interface.implementer(IFireplace)
class Fireplace(grokcore.site.LocalUtility):
    pass


@interface.implementer(IClub)
class Club(object):
    pass


@interface.implementer(IClub, ISpiky)
class SpikyClub(object):
    pass


@interface.implementer(IMammoth, IClub)
class Mammoth(grokcore.site.LocalUtility):
    pass


@interface.implementer(IMammoth, IClub)
class SabretoothTiger(grokcore.site.LocalUtility):
    grokcore.site.provides(IMammoth)


class IPainting(persistent.interfaces.IPersistent):
    pass


@interface.implementer(IPainting)
class CavePainting(grokcore.site.LocalUtility):
    pass


@interface.implementer(IPainting)
class ColoredCavePainting(grokcore.site.LocalUtility):
    grokcore.site.provides(IPainting)


class Cave(grokcore.site.Site):
    grokcore.site.local_utility(Fireplace)
    grokcore.site.local_utility(Club)
    grokcore.site.local_utility(SpikyClub, provides=IClub, name='spiky')
    grokcore.site.local_utility(Mammoth, provides=IMammoth)
    grokcore.site.local_utility(SabretoothTiger, name='tiger')
    grokcore.site.local_utility(
        CavePainting, name='blackandwhite', provides=IPainting)
    grokcore.site.local_utility(ColoredCavePainting, name='color')



"""
Local Utilities can be registered on subclasses of grok.Site using
grok.local_utility:

  >>> cave = SpikyCave()
  >>> getRootFolder()['cave'] = cave

  >>> from zope import component
  >>> from zope.component.hooks import getSite, setSite
  >>> setSite(cave)

  >>> club = component.getUtility(IClub)
  >>> IClub.providedBy(club)
  True
  >>> isinstance(club, SpikyClub)
  True

  >>> names = list(cave.getSiteManager().keys())
  >>> len(names)
  1

  >>> print(names[0])
  SpikyClub

"""
import grokcore.site
from zope import interface


class IClub(interface.Interface):
    pass


@interface.implementer(IClub)
class Club(grokcore.site.LocalUtility):
    pass


@interface.implementer(IClub)
class SpikyClub(grokcore.site.LocalUtility):
    pass


class Cave(grokcore.site.Site):
    grokcore.site.local_utility(Club)


class SpikyCave(Cave):
    grokcore.site.local_utility(SpikyClub)



##############################################################################
#
# Copyright (c) 2006-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from zope.interface import Interface, Attribute, implementer
from zope.interface.interfaces import IObjectEvent
from grokcore.component.interfaces import IGrokcoreComponentAPI


class IApplication(Interface):
    """Interface to mark the local site used as application root.
    """


class IApplicationAddedEvent(IObjectEvent):
    """A Grok Application has been added with success.

    This event can be used to trigger the creation of contents or other tasks
    that require the application to be fully there : utilities installed
    and indexes created in the catalog."""


@implementer(IApplicationAddedEvent)
class ApplicationAddedEvent(object):
    """A Grok Application has been added.
    """

    def __init__(self, app):
        assert IApplication.providedBy(app)
        self.object = app


class IUtilityInstaller(Interface):
    """This install an utility in a site. Let you have different
    'installation' method if you want (one for Zope2 / Zope3).
    """

    def __call__(site, utility, provides, name=u'',
                 name_in_container=None, public=False, setup=None):
        """Setup an utility.
        """


class IBaseClasses(Interface):
    Site = Attribute("Mixin class for sites.")

    LocalUtility = Attribute("Base class for local utilities.")

    Application = Attribute("Base class for applications.")


class IDirectives(Interface):
    def local_utility(factory, provides=None, name=u'',
                      setup=None, public=False, name_in_container=None):
        """Register a local utility.

        factory - the factory that creates the local utility
        provides - the interface the utility should be looked up with
        name - the name of the utility
        setup - a callable that receives the utility as its single argument,
                it is called after the utility has been created and stored
        public - if False, the utility will be stored below ++etc++site
                 if True, the utility will be stored directly in the site.
                 The site should in this case be a container.
        name_in_container - the name to use for storing the utility
        """

    def install_on(event):
        """Explicitly specify when a local utility will be installed.
        """

    def provides(interface):
        """Explicitly specify with which interface a component will be
        looked up."""


class IGrokcoreSiteAPI(IGrokcoreComponentAPI, IBaseClasses, IDirectives):
    """grokcore.site's public API."""

    IApplication = Attribute('The application model interface')

    IApplicationAddedEvent = Attribute(
        'The application initialized event interface')

    ApplicationAddedEvent = Attribute(
        'The application initialized event factory')

    def getSite():
        """Get the current site."""

    def getApplication():
        """Return the nearest enclosing `grok.Application`."""



##############################################################################
#
# Copyright (c) 2006-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from zope.lifecycleevent.interfaces import IObjectAddedEvent
from zope.lifecycleevent.interfaces import IObjectRemovedEvent
from zope.site import LocalSiteManager
from zope.site.site import _findNextSiteManager

import grokcore.component
from grokcore.site.components import Site


@grokcore.component.subscribe(Site, IObjectAddedEvent)
def addSiteHandler(site, event):
    """Add a local site manager to a Grok site object upon its creation.
    """
    if event.oldParent is not None:
        return
    sitemanager = LocalSiteManager(site)
    del sitemanager['default']
    site.setSiteManager(sitemanager)


@grokcore.component.subscribe(Site, IObjectRemovedEvent)
def removeSiteHandler(site, event):
    """Cleanup a after a site was removed.
    """
    if event.newParent is not None:
        # Please note that the code for moving a site is properly implemented
        # inside zope.site.
        return
    local = site.getSiteManager()
    parent = _findNextSiteManager(site)
    if parent is not None:
        parent.removeSub(local)



import doctest
import unittest
from pkg_resources import resource_listdir

from zope.app.appsetup.testlayer import ZODBLayer
from zope.testing import renormalizing

import grokcore.site

FunctionalLayer = ZODBLayer(grokcore.site)


checker = renormalizing.RENormalizing([])


def suiteFromPackage(name):
    layer_dir = 'functional'
    files = resource_listdir(__name__, '{}/{}'.format(layer_dir, name))
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename == '__init__.py':
            continue

        dottedname = 'grokcore.site.tests.%s.%s.%s' % (
            layer_dir, name, filename[:-3])
        test = doctest.DocTestSuite(
            dottedname,
            checker=checker,
            extraglobs=dict(getRootFolder=FunctionalLayer.getRootFolder),
            optionflags=(
                doctest.ELLIPSIS +
                doctest.NORMALIZE_WHITESPACE +
                doctest.REPORT_NDIFF +
                renormalizing.IGNORE_EXCEPTION_MODULE_IN_PYTHON2))
        test.layer = FunctionalLayer

        suite.addTest(test)
    return suite


def test_suite():
    suite = unittest.TestSuite()
    for name in ['utility', 'site', 'application']:
        suite.addTest(suiteFromPackage(name))
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from setuptools import setup, find_packages
import os


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


long_description = (
    read('README.rst')
    + '\n' +
    read('CHANGES.rst')
    )

tests_require = [
    'zope.app.appsetup',
    'zope.component',
    'zope.configuration',
    'zope.testing',
    'grokcore.content',
    ]

setup(
    name='grokcore.site',
    version='3.1',
    author='Grok Team',
    author_email='grok-dev@zope.org',
    url='http://grok.zope.org',
    download_url='http://pypi.python.org/pypi/grokcore.site',
    description='Grok-like configuration for Zope local site and utilities',
    long_description=long_description,
    license='ZPL',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Framework :: Zope :: 3',
    ],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['grokcore'],
    include_package_data=True,
    zip_safe=False,
    install_requires=['setuptools',
                      'grokcore.component >= 2.1',
                      'martian >= 0.13',
                      'persistent',
                      'zope.annotation',
                      'zope.component',
                      'zope.container',
                      'zope.event',
                      'zope.interface',
                      'zope.lifecycleevent',
                      'zope.schema',
                      'zope.site',
                      ],
    tests_require=tests_require,
    extras_require={'test': tests_require},
)



"""
By default, a utility is not in the public site; it's in ++etc++site. We can
also specify the utility to be public. It will then be created in the container
that is the site. The name the utility should have in the container can
be controlled using name_in_container:

  >>> cave = Cave()
  >>> getRootFolder()["cave"] = cave

  >>> from zope import component
  >>> from zope.component.hooks import getSite, setSite
  >>> setSite(cave)
  >>> cave['fireplace'] is component.getUtility(IFireplace)
  True

name_in_container can also be used for objects stored under the site manager
(that is in ++etc++site):

   >>> cave2 = Cave2()
   >>> getRootFolder()['cave2'] = cave2
   >>> setSite(cave2)
   >>> (cave2.getSiteManager()['fireplace'] is
   ...  component.getUtility(IFireplace))
   True

"""

import grokcore.site
from zope import interface
from zope.container.btree import BTreeContainer


class IFireplace(interface.Interface):
    pass


@interface.implementer(IFireplace)
class Fireplace(grokcore.site.LocalUtility):
    pass


class Cave(BTreeContainer, grokcore.site.Site):
    grokcore.site.local_utility(Fireplace, public=True,
                                name_in_container='fireplace')


class Cave2(BTreeContainer, grokcore.site.Site):
    grokcore.site.local_utility(Fireplace, public=False,
                                name_in_container='fireplace')



"""
An application is a mixin for grok application objects.

You can get the current application by using the
grok.getApplication() function. Typically this will return the same
object as grok.getSite(), but it is possible to have sub-Site objects
which will be returned for grok.getSite(), where-as grok.getApplication
will walk up the tree until it reaches the top-level site object.

Let's create an application, then get it using grok.getApplication():

  >>> import grokcore.site
  >>> import zope.component.hooks
  >>> root = getRootFolder()

  >>> app = grokcore.site.util.create_application(Cave, root, 'mycave')
  >>> root['cave'] = app
  >>> zope.component.hooks.setSite(app)
  >>> grokcore.site.getApplication()
  <grokcore.site.tests.functional.application.application.Cave object at ...>

Or get it using getSite():

  >>> from zope.component.hooks import getSite
  >>> getSite()
  <grokcore.site.tests.functional.application.application.Cave object at ...>

Now we can create a container with a sub-site. When we call grok.getSite()
we'll get the box:

  >>> root['cave']['box'] = WoodBox()
  >>> zope.component.hooks.setSite(root['cave']['box'])
  >>> getSite()
  <grokcore.site.tests.functional.application.application.WoodBox
  object at ...>

But when we call grokcore.site.util.getApplication() we get the cave:

  >>> grokcore.site.getApplication()
  <grokcore.site.tests.functional.application.application.Cave object at ...>

If you try to create an application that is not valid it will fail:

  >>> grokcore.site.util.create_application(object(), root, 'myobject')
  Traceback (most recent call last):
    ...
  zope.schema._bootstrapinterfaces.WrongType: ...

"""
import grokcore.content
import grokcore.site


class Cave(grokcore.content.Container, grokcore.site.Application):
    """A shelter for homeless cavemen.
    """


class WoodBox(grokcore.content.Container, grokcore.site.Site):
    """A prehistoric container for holding ZCA registries.
    """



import doctest
import re
import unittest
from pkg_resources import resource_listdir

from zope.testing import cleanup, renormalizing
import zope.component.eventtesting

checker = renormalizing.RENormalizing([
    # str(Exception) has changed from Python 2.4 to 2.5 (due to
    # Exception now being a new-style class).  This changes the way
    # exceptions appear in traceback printouts.
    (re.compile(
        r"ConfigurationExecutionError: <class '([\w.]+)'>:"),
        r'ConfigurationExecutionError: \1:')])


def setUpZope(test):
    zope.component.eventtesting.setUp(test)


def cleanUpZope(test):
    cleanup.cleanUp()


def suiteFromPackage(name):
    layer_dir = 'base'
    files = resource_listdir(__name__, '{}/{}'.format(layer_dir, name))
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename.endswith('_fixture.py'):
            continue
        if filename == '__init__.py':
            continue

        dottedname = 'grokcore.site.tests.%s.%s.%s' % (
            layer_dir, name, filename[:-3])
        test = doctest.DocTestSuite(
            dottedname,
            setUp=setUpZope,
            tearDown=cleanUpZope,
            checker=checker,
            optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)

        suite.addTest(test)
    return suite


def test_suite():
    suite = unittest.TestSuite()
    for name in ['utility', 'application']:
        suite.addTest(suiteFromPackage(name))
    return suite


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""
Local Utilities can be registered on subclasses of grok.Site using
grok.local_utility but only on grok.install_on:

  >>> cave = Cave()
  >>> getRootFolder()['cave'] = cave

  >>> from zope.component import queryUtility
  >>> from zope.component.hooks import setSite
  >>> from zope.event import notify
  >>> setSite(cave)

  >>> club = queryUtility(IClub)
  >>> club is None
  True

  >>> notify(PartyEvent(cave))
  >>> club = queryUtility(IClub)
  >>> IClub.providedBy(club)
  True
"""
import grokcore.site
from zope.interface import Interface, implementer
from zope.interface.interfaces import ObjectEvent, IObjectEvent


class IPartyEvent(IObjectEvent):
    pass


@implementer(IPartyEvent)
class PartyEvent(ObjectEvent):
    pass


class IClub(Interface):
    pass


@implementer(IClub)
class Club(grokcore.site.LocalUtility):
    pass


class Cave(grokcore.site.Site):
    grokcore.site.install_on(IPartyEvent)
    grokcore.site.local_utility(Club)
